@@ -5,26 +5,27 @@
 #' @param progress boolean: display a progress bar
 #'
 #' @details
-#' The United Nations \href{https://www.dhsprogram.com/}{Demographic and Health Surveys} (DHS) program regularly collects health data from
-#'    population-representative samples in many countries using standardized surveys since 1984. The "individual
-#'    recode" data files contain women's responses, and are available in SPSS, SAS, and Stata formats
-#'    from \href{https://www.dhsprogram.com/}{https://www.dhsprogram.com/}. Access to these data requires
-#'    a \href{https://dhsprogram.com/data/Access-Instructions.cfm}{free application}. The `dhs()` function
-#'    reads one or more of these files, extracts and recodes selected variables useful for studying childfree
-#'    adults and other family statuses, then returns a single data frame.
+#' The \href{https://www.dhsprogram.com/}{Demographic and Health Surveys} (DHS) program regularly collects
+#'    health data from population-representative samples in many countries using standardized surveys since 1984. The
+#'    "individual recode" data files contain women's responses, while the "men recode" files contain men's responses. These
+#'    files are available in SPSS, SAS, and Stata formats from \href{https://www.dhsprogram.com/}{https://www.dhsprogram.com/},
+#'    however access requires a \href{https://dhsprogram.com/data/Access-Instructions.cfm}{free application}. The `dhs()` function
+#'    reads one or more of these files, extracts and recodes selected variables useful for studying childfree adults and other
+#'    family statuses, then returns a single data frame.
 #'
 #' Although access to DHS data requires an application, the DHS program provides \href{https://dhsprogram.com/data/Download-Model-Datasets.cfm}{model datasets}
-#'    for practice. The example provided below uses the model data file "ZZIR62FL.SAV", which contains
-#'    fictitious data, but has the same structure as real DHS data files. The example can be run without
-#'    prior application for data access.
+#'    for practice. The example provided below uses the model data file "ZZIR62FL.SAV", which contains fictitious women's data,
+#'    but has the same structure as real DHS data files. The example can be run without prior application for data access.
 #'
 #' **Known issues**
-#'   * The SPSS-formatted files containing data from Gabon Recode 4 (GAIR41FL.SAV) and Turkey Recode 4 (TRIR41FL.SAV)
-#'     contain encoding errors. Use the SAS-formatted files (GAIR41FL.SAS7BDAT and TRIR41FL.SAS7BDAT) instead.
-#'   * In some cases, DHS makes available individual recode data files for specific states. For example, data from Ondo
-#'     State in Nigeria from Wave 1 is contained in OSIR01FL.SAV, data from states in India from 1999 are contained in
-#'     files named XXIR42FL.SAV, where the "XX" is a two-letter state code. This function only accepts whole-country
-#'     individual recode data files, and not these state-specific data files.
+#'   * The SPSS-formatted files containing data from Gabon Recode 4 (GAIR41FL.SAV, GAMR41FL.SAV) and Turkey Recode 4 (TRIR41FL.SAV, TRMR41FL.SAV)
+#'     contain encoding errors. Use the SAS-formatted files (GAIR41FL.SAS7BDAT, GAMR41FL.SAS7BDAT, TRIR41FL.SAS7BDAT, TRMR41FL.SAS7BDAT) instead.
+#'   * In some cases, DHS makes available individual recode data files for specific regions. For example, women's data from individual states
+#'     in India from 1999 are contained in files named XXIR42FL.SAV, where the "XX" is a two-letter state code. The `dhs()` function has only
+#'     been tested using whole-country files, and may not perform as expected for regional files.
+#'   * Variables containing women's responses in the individual recode files begin with `v`, while variables containing men's responses in the
+#'     men recode files begin with `mv`. When applying `dhs()` to both female and male data, these are automatically harmonized. However, if
+#'     extra variables are requested using the `extra.vars` option, be sure to specify both names (e.g. `extra.vars = c("v201", "mv201")`).
 #'
 #' @return A data frame containing variables described in the codebook available using \code{vignette("codebooks")}
 #'
@@ -34,6 +35,8 @@
 #' \donttest{data <- dhs(files = c("ZZIR62FL.SAV"), extra.vars = c("v201"))}
 dhs <- function(files, extra.vars = NULL, progress = TRUE) {
 
+  if (length(files) > 1 & "ZZIR62FL.SAV" %in% files) {stop("Model data (file ZZIR62FL.SAV) should not be combined with files containing real data.")}
+
   if (!is.null(extra.vars)) {extra.vars <- tolower(extra.vars)}  #Make requested extra variables lowercase
 
   if (progress) {message("Processing DHS data files -")}
@@ -46,26 +49,33 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
     if (progress) {utils::setTxtProgressBar(pb,file)}
 
     #Import raw data
-    if (files=="ZZIR62FL.SAV") {  #Model file from https://dhsprogram.com/data/Download-Model-Datasets.cfm
+    if (files[file]=="ZZIR62FL.SAV") {  #Model file from https://dhsprogram.com/data/Download-Model-Datasets.cfm
       temp <- tempfile()
       utils::download.file(url = "https://osf.io/download/hk23e", destfile = temp)
       dat <- rio::import(temp, format = "sav")
-    }
-    if (files!="ZZIR62FL.SAV") {dat <- rio::import(files[file])}
+    } else {dat <- rio::import(files[file])}
     colnames(dat) <- tolower(colnames(dat))  #Make all variables lowercase
 
+    #Check type of file
+    female <- NULL
+    if (colnames(dat)[1]=="caseid" | colnames(dat)[1]=="case$id") {female <- TRUE}
+    if (colnames(dat)[1]=="mcaseid" | colnames(dat)[1]=="mcase$id") {female <- FALSE}
+    if (is.null(female)) {stop(paste0(files[file], " does not appear to be an individual or men DHS recode file."))}
+
     #### Family Status ####
     #Number of children
-    dat$numkid <- dat$v201
+    if (female) {dat$numkid <- dat$v201} else {dat$numkid <- dat$mv201}
 
     #Want children
-    dat$want <- dat$v602
+    if (female) {dat$want <- dat$v602} else {dat$want <- dat$mv602}
+    dat$want[which(dat$want==0)] <- NA  #Zero doesn't seem to be a valid value
     dat$want[which(dat$want==4)] <- 5  #Combine 4-Sterilized with 5-Infecund
+    dat$want[which(dat$want==7)] <- 5  #Combine 7-Man Infecund with 5-Infecund
     dat$want[which(dat$want>=6)] <- NA  #Various labels, none about wants
     dat$want <- factor(dat$want, levels = c(1,2,3,5), labels = c("Have (another)", "Undecided", "No (more)", "Infecund"))
 
     #Ideal number of children
-    dat$ideal <- dat$v613
+    if (female) {dat$ideal <- dat$v613} else {dat$ideal <- dat$mv613}
     dat$ideal[which(dat$ideal==98)] <- -1  #Special code for "Don't Know"
     dat$ideal[which(dat$ideal>30)] <- NA  #Treat all values above 30 as missing (includes some undocumented country-specific special codes)
 
@@ -128,38 +138,45 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
 
     #### Demographics ####
     #Sex
-    dat$sex <- 1
+    if (female) {dat$sex <- 1} else {dat$sex <- 2}
     dat$sex <- factor(dat$sex, levels = c(1,2,3), labels = c("Female", "Male", "Other"))
 
     #Age in years
-    dat$age <- dat$v012
+    if (female) {dat$age <- dat$v012} else {dat$age <- dat$mv012}
 
     #Education in years
-    dat$education <- dat$v133
+    if (female) {dat$education <- dat$v133} else {dat$education <- dat$mv133}
     dat$education[dat$education>=40] <- NA
 
     #Partnership status
-    dat$partnered <- dat$v502 + 1
+    if (female) {dat$partnered <- dat$v502 + 1} else {dat$partnered <- dat$mv502 + 1}
     dat$partnered <- factor(dat$partnered, levels = c(1,2,3), labels = c("Never", "Currently", "Formerly"))
 
     #Residence
-    dat$residence <- dat$v102
-    if (dat$v000[1]=="MX" & dat$v007[1]==87) {  #Mexico Wave 1 used a different coding
-      dat$residence[which(dat$residence<4)] <- 2  #Code as rural (1) Less than 2500, (2) 2500-19999, and (3) 20000+
-      dat$residence[which(dat$residence==4)] <- 1  #Code as urban (4) Areas Metropolitanas
+    if (female) {
+      dat$residence <- dat$v102
+      if (dat$v000[1]=="MX" & dat$v007[1]==87) {  #Mexico Wave 1 used a different coding
+        dat$residence[which(dat$residence<4)] <- 2  #Code as rural (1) Less than 2500, (2) 2500-19999, and (3) 20000+
+        dat$residence[which(dat$residence==4)] <- 1  #Code as urban (4) Areas Metropolitanas
+      }
+    }
+    if (!female) {
+      dat$residence <- dat$mv102
+      if (dat$mv000[1]=="MX" & dat$mv007[1]==87) {  #Mexico Wave 1 used a different coding
+        dat$residence[which(dat$residence<4)] <- 2  #Code as rural (1) Less than 2500, (2) 2500-19999, and (3) 20000+
+        dat$residence[which(dat$residence==4)] <- 1  #Code as urban (4) Areas Metropolitanas
+      }
     }
     dat$residence <- factor(dat$residence, levels = c(2,98,99,1), labels = c("Rural", "Town", "Suburb", "Urban"), ordered = TRUE)  #98 and 99 are dummy values; "Town" and "Suburb" categories are not used by DHS
 
     #Employed
-    dat$employed <- NA
-    dat$employed[which(dat$v714==0)] <- 0  #Not employed
-    dat$employed[which(dat$v714==1)] <- 1  #Employed
+    if (female) {dat$employed <- dat$v714} else {dat$employed <- dat$mv714}
 
     #### Attitude ####
     #Religion
     dat$religion <- NA
 
-    x <- as.data.frame(attr(dat$v130, "labels"))  #Get file-specific dictionary (o = old value, l = label, n = new value)
+    if (female) {x <- as.data.frame(attr(dat$v130, "labels"))} else {x <- as.data.frame(attr(dat$mv130, "labels"))}  #Get file-specific dictionary (o = old value, l = label, n = new value)
     if (nrow(x) > 0) {  #If there are labeled values for religions...
       x$label <- rownames(x)
       colnames(x) <- c("o", "l")
@@ -219,7 +236,7 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
 
         if (x$l[i] %in% c("Hindu", "Hinduism")) {x$n[i] <- 8}  #Hindu
 
-        dat$religion[which(dat$v130==x$o[i])] <- x$n[i]  #Insert new value into recoded religion variable
+        if (female) {dat$religion[which(dat$v130==x$o[i])] <- x$n[i]} else {dat$religion[which(dat$mv130==x$o[i])] <- x$n[i]}  #Insert new value into recoded religion variable
       }
     }
 
@@ -228,7 +245,8 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
 
     #### Design ####
     #Identifier (non-standard variable name in Egypt 1988-89)
-    if (dat$v000[1]=="EG" & (dat$v007[1]==88 | dat$v007[1]==89)) {dat$id <- dat$`case$id`} else {dat$id <- dat$caseid}
+    if (female) {if (dat$v000[1]=="EG" & (dat$v007[1]==88 | dat$v007[1]==89)) {dat$id <- dat$`case$id`} else {dat$id <- dat$caseid}}
+    if (!female) {if (dat$mv000[1]=="EG" & (dat$mv007[1]==88 | dat$mv007[1]==89)) {dat$id <- dat$`mcase$id`} else {dat$id <- dat$mcaseid}}
     dat$id <- as.character(dat$id)
 
     #Country
@@ -236,33 +254,38 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
                        "CD", "CI", "DR", "EC", "EG", "ES", "EK", "ER", "ET", "GA", "GM", "GH", "GU", "GN", "GY", "HT", "HN", "IA", "ID", "JO",
                        "KK", "KE", "KY", "LA", "LS", "LB", "MD", "MW", "MV", "ML", "MR", "MX", "MB", "MA", "MZ", "MM", "NM", "NP", "NC", "NI",
                        "NG", "OS", "PK", "PY", "PE", "PH", "RW", "WS", "ST", "SN", "SL", "ZA", "LK", "SD", "SZ", "TJ", "TZ", "TH", "TL", "TG",
-                       "TT", "TN", "TR", "TM", "UG", "UA", "UZ", "VN", "YE", "ZM", "ZW", "PG")
+                       "TT", "TN", "TR", "TM", "UG", "UA", "UZ", "VN", "YE", "ZM", "ZW", "PG", "ZZ")
     country.names <- c("Afghanistan", "Albania", "Angola", "Armenia", "Azerbaijan", "Bangladesh", "Benin", "Bolivia", "Botswana", "Brazil", "Burkina Faso", "Burundi", "Cambodia", "Cameroon", "Cape Verde", "Central African Republic", "Chad", "Columbia", "Comoros", "Congo",
                        "Congo Democratic Republic", "Cote d'Ivoire", "Dominican Republic", "Ecuador", "Egypt", "El Salvador", "Equatorial Guinea", "Eritrea", "Ethiopia", "Gabon", "Gambia", "Ghana", "Guatamala", "Guinea", "Guyana", "Haiti", "Honduras", "India", "Indonesia", "Jordan",
                        "Kazakhstan", "Kenya", "Kyrgyz Republic", "Lao People's Democratic Republic", "Lesotho", "Liberia", "Madagascar", "Malawi", "Maldives", "Mali", "Mauritania", "Mexico", "Moldova", "Morocco", "Mozambique", "Myanmar", "Namibia", "Nepal", "Nicaragua", "Niger",
                        "Nigeria", "Nigeria (Ondo State)", "Pakistan", "Paraguay", "Peru", "Philippines", "Rwanda", "Samoa", "Sao Tome and Principe", "Senegal", "Sierra Leone", "South Africa", "Sri Lanka", "Sudan", "Swaziland", "Tajikstan", "Tanzania", "Thailand", "Timor-Leste", "Togo",
-                       "Trinidad and Tobago", "Tunisia", "Turkey", "Turkministan", "Uganda", "Ukraine", "Uzbekistan", "Vietnam", "Yemen", "Zambia", "Zimbabwe", "Papua New Guinea")
-    dat$country <- country.names[match(substr(dat$v000,1,2), country.codes)]
+                       "Trinidad and Tobago", "Tunisia", "Turkey", "Turkministan", "Uganda", "Ukraine", "Uzbekistan", "Vietnam", "Yemen", "Zambia", "Zimbabwe", "Papua New Guinea", "Fake Country")
+    if (female) {dat$country <- country.names[match(substr(dat$v000,1,2), country.codes)]} else {dat$country <- country.names[match(substr(dat$mv000,1,2), country.codes)]}
 
     #Sampling weight
-    dat$weight <- dat$v005/1000000  #Sampling weight
+    if (female) {dat$weight <- dat$v005/1000000} else {dat$weight <- dat$mv005/1000000}
 
     #Wave (called "Recode" in the DHS)
-    dat$wave <- as.numeric(substr(dat$v000,3,3))  #Recode
+    if (female) {
+      dat$wave <- as.numeric(substr(dat$v000,3,3))
+      dat$wave[which(dat$country=="Vietnam" & dat$v007==97)] <- 3  #Recode was labeled as "T" for Vietnam 1997
+      dat$wave[which(dat$country=="Vietnam" & dat$v007==2)] <- 4  #Recode was labeled as "T" for Vietnam 2002
+    }
+    if (!female) {
+      dat$wave <- as.numeric(substr(dat$mv000,3,3))
+      dat$wave[which(dat$country=="Vietnam" & dat$mv007==97)] <- 3  #Recode was labeled as "T" for Vietnam 1997
+      dat$wave[which(dat$country=="Vietnam" & dat$mv007==2)] <- 4  #Recode was labeled as "T" for Vietnam 2002
+    }
     dat$wave[which(is.na(dat$wave))] <- 1  #In recode 1, v000 only contained the country code
-    dat$wave[which(dat$country=="Vietnam" & dat$v007==97)] <- 3  #Recode was labeled as "T" for Vietnam 1997
-    dat$wave[which(dat$country=="Vietnam" & dat$v007==2)] <- 4  #Recode was labeled as "T" for Vietnam 2002
 
-    #Year of data collection
-    dat$year <- dat$v007
-    dat$year[which(dat$year>=85 & dat$year<=99 & dat$country!="Nepal")] <- dat$year[which(dat$year>=85 & dat$year<=99 & dat$country!="Nepal")] + 1900  #Fix two-digit years
-    dat$year[which(dat$year>=0 & dat$year<=10 & dat$country!="Nepal")] <- dat$year[which(dat$year>=0 & dat$year<=10 & dat$country!="Nepal")] + 2000  #Fix two-digit years
-    dat$year[which(dat$year>1900 & dat$country=="Nepal")] <- dat$year[which(dat$year>1900 & dat$country=="Nepal")] - 57  #Fix Nepali years
-    dat$year[which(dat$year<100 & dat$country=="Nepal")] <- dat$year[which(dat$year<100 & dat$country=="Nepal")] + 1943
-    dat$year[which(dat$country=="Afghanistan" & dat$wave==7)] <- 2015  #The 2015 Afghanistan used year from Afghan calendar
-
-    #Month of data collection
-    dat$month <- dat$v006
+    #Year and month of data collection
+    if (female) {dat$cmc <- dat$v008} else {dat$cmc <- dat$mv008}  #Century month code
+    if (dat$country[1]=="Ethiopia") {dat$cmc <- dat$cmc + 92}  #Correction for Ethiopian calendar (https://dhsprogram.com/data/Guide-to-DHS-Statistics/Organization_of_DHS_Data.htm)
+    if (dat$country[1]=="Nepal" & dat$wave[1]!=3) {dat$cmc <- dat$cmc - 681}  #Correction for Nepali calendar (https://dhsprogram.com/data/Guide-to-DHS-Statistics/Organization_of_DHS_Data.htm)
+    if (dat$country[1]=="Nepal" & dat$wave[1]==3) {dat$cmc <- dat$cmc + 519}
+    if (dat$country[1]=="Afghanistan") {dat$cmc <- dat$cmc + 255}  #Correction for Afghani calendar (https://dhsprogram.com/data/Guide-to-DHS-Statistics/Organization_of_DHS_Data.htm)
+    dat$year <- 1900+floor((dat$cmc-1)/12)
+    dat$month <- dat$cmc - (12 * (dat$year - 1900))
     dat$month <- factor(dat$month, levels = c(1:12), labels = c("January", "February", "March", "April", "May", "June",
                                                                 "July", "August", "September", "October", "November", "December"),
                         ordered = TRUE)
